@@ -584,7 +584,7 @@ seqApply <- function(gdsfile, var.name, FUN,
     .useraw=FALSE, .progress=FALSE, .list_dup=TRUE, .balancing=FALSE, ...)
 {
     # check
-    stopifnot(inherits(gdsfile, "SeqVarGDSClass"))
+    stopifnot(inherits(gdsfile, "SeqVarGDSClass") || is.character(gdsfile))
     stopifnot(is.character(var.name), length(var.name)>0L)
     FUN <- match.fun(FUN)
     margin <- match.arg(margin)
@@ -595,16 +595,25 @@ seqApply <- function(gdsfile, var.name, FUN,
         length(.progress)==1L)
     stopifnot(is.logical(.balancing), length(.balancing)==1L)
 
+    # gds file
+    if (is.character(gdsfile))
+    {
+        stopifnot(length(gdsfile)==1L)
+        if (isTRUE(.progress))
+            .cat("Open ", sQuote(basename(gdsfile)))
+        gdsfile <- seqOpen(gdsfile, allow.duplicate=TRUE)
+        on.exit(seqClose(gdsfile))
+    }
+
     parallel <- .McoreParallel(parallel)
     njobs <- .NumParallel(parallel)
-
     param <- list(useraw=.useraw, list_dup=.list_dup,
         progress=isTRUE(.progress), progressfile=NULL)
     if (is.character(.progress) && njobs==1L)
     {
         param$progress <- isTRUE(attr(.progress, "verbose"))
         param$progressfile <- file(.progress, "at")
-        on.exit(close(param$progressfile))
+        on.exit(close(param$progressfile), add=TRUE)
         if (isTRUE(attr(.progress, "delete")))
             on.exit(unlink(.progress, force=TRUE), add=TRUE)
     }
@@ -679,7 +688,7 @@ seqBlockApply <- function(gdsfile, var.name, FUN, margin=c("by.variant"),
     .progress=FALSE, ...)
 {
     # check
-    stopifnot(inherits(gdsfile, "SeqVarGDSClass"))
+    stopifnot(inherits(gdsfile, "SeqVarGDSClass") || is.character(gdsfile))
     stopifnot(is.character(var.name), length(var.name)>0L)
     FUN <- match.fun(FUN)
     margin <- match.arg(margin)
@@ -693,12 +702,19 @@ seqBlockApply <- function(gdsfile, var.name, FUN, margin=c("by.variant"),
 
     parallel <- .McoreParallel(parallel)
     njobs <- .NumParallel(parallel)
+    if (!inherits(as.is, "connection") & !inherits(as.is, "gdsn.class"))
+        as.is <- match.arg(as.is)
     param <- list(bsize=bsize, useraw=.useraw, padNA=.padNA, tolist=.tolist,
         progress=.progress, progressfile=NULL)
 
-    if (!inherits(as.is, "connection") & !inherits(as.is, "gdsn.class"))
+    # gds file
+    if (is.character(gdsfile))
     {
-        as.is <- match.arg(as.is)
+        stopifnot(length(gdsfile)==1L)
+        if (isTRUE(.progress))
+            .cat("Open ", sQuote(basename(gdsfile)))
+        gdsfile <- seqOpen(gdsfile, allow.duplicate=TRUE)
+        on.exit(seqClose(gdsfile))
     }
 
     # get # of samples & variants
@@ -1127,11 +1143,13 @@ seqAlleleCount <- function(gdsfile, ref.allele=0L, minor=FALSE,
 # Get AF/MAF, AC/MAC and missing rate for variants
 #
 
-seqGetAF_AC_Missing <- function(gdsfile, minor=FALSE, parallel=seqGetParallel(),
-    balancing=NA, verbose=FALSE)
+seqGetAF_AC_Missing <- function(gdsfile, minor=FALSE, alt=FALSE, ns=FALSE,
+    parallel=seqGetParallel(), balancing=NA, verbose=FALSE)
 {
     # check
     stopifnot(is.logical(minor), length(minor)==1L)
+    stopifnot(is.logical(alt), length(alt)==1L)
+    stopifnot(is.logical(ns), length(ns)==1L)
     stopifnot(is.logical(balancing), length(balancing)==1L)
     stopifnot(is.logical(verbose), length(verbose)==1L)
     parallel <- .McoreParallel(parallel)
@@ -1149,11 +1167,12 @@ seqGetAF_AC_Missing <- function(gdsfile, minor=FALSE, parallel=seqGetParallel(),
     }
 
     # check genotypes
+    dm <- .seldim(gdsfile)
     gv <- .has_geno(gdsfile)
     if (gv)
     {
         nm <- "genotype"
-        ploidy <- .dim(gdsfile)[1L]
+        ploidy <- dm[1L]
     } else {
         nm <- .has_dosage(gdsfile)
         ploidy <- getOption("seqarray.ploidy", 2L)
@@ -1162,21 +1181,26 @@ seqGetAF_AC_Missing <- function(gdsfile, minor=FALSE, parallel=seqGetParallel(),
 
     # calculate
     m3s <- seqParallel(parallel, gdsfile, split="by.variant", .combine="list",
-        FUN = function(f, pg, nm, pl, minor, cn)
+        FUN = function(f, pg, nm, pl, minor, alt, cn)
         {
             m3 <- matrix(0, nrow=3L, ncol=.seldim(f)[3L])
-            .cfunction3("FC_AF_AC_MISS_Init")(m3, pl, minor)
+            .cfunction4("FC_AF_AC_MISS_Init")(m3, pl, minor, alt)
             seqApply(f, nm, as.is="none", FUN=.cfunction(cn), .useraw=NA,
                 .progress=.process_verbose(pg))
             m3
         }, .balancing=balancing, .status_file=TRUE, .proc_time=verbose,
-            pg=verbose, nm=nm, pl=ploidy, minor=minor,
+            pg=verbose, nm=nm, pl=ploidy, minor=minor, alt=alt,
             cn=ifelse(gv, "FC_AF_AC_MISS_Geno", "FC_AF_AC_MISS_DS"))
 
     # merge
     if (is.list(m3s)) m3s <- do.call(cbind, m3s)
+    rv <- data.frame(af=m3s[1L,], ac=m3s[2L,], miss=m3s[3L,])
+    if (gv) rv$ac <- as.integer(rv$ac)
+    if (isTRUE(ns))
+        rv$ns <- as.integer(round(dm[2L]*(1L-rv$miss)))
+
     # output
-    data.frame(af=m3s[1L,], ac=m3s[2L,], miss=m3s[3L,])
+    rv
 }
 
 
