@@ -2,7 +2,7 @@
 //
 // Index.h: Indexing Objects
 //
-// Copyright (C) 2016-2022    Xiuwen Zheng
+// Copyright (C) 2016-2025    Xiuwen Zheng
 //
 // This file is part of SeqArray.
 //
@@ -76,6 +76,8 @@ class ErrSeqArray;
 template<typename TYPE> class COREARRAY_DLL_LOCAL C_RLE
 {
 public:
+	friend class CChromIndex;
+
 	/// constructor
 	C_RLE()
 	{
@@ -249,15 +251,15 @@ public:
 
 	typedef vector<TRange> TRangeList;
 
+	/// map to TRangeList from chromosome coding
+	map<string, TRangeList> Map;
+
 	/// constructor
 	CChromIndex();
-
 	/// clear
 	void Clear();
-
 	/// represent chromosome codes as a RLE object in Map
 	void AddChrom(PdGDSFolder Root);
-
 	/// the total length of a TRangeList object
 	size_t RangeTotalLength(const TRangeList &RngList);
 
@@ -265,9 +267,10 @@ public:
 	inline bool Empty() const { return Map.empty(); }
 	/// return chromosome coding with the index 
 	inline const string &operator [](size_t idx) { return RleChr[idx]; }
-
-	/// map to TRangeList from chromosome coding
-	map<string, TRangeList> Map;
+	/// return the RLE representation of chromosome: value
+	inline vector<string> &RLE_Values() { return RleChr.Values; }
+	/// return the RLE representation of chromosome: length
+	inline vector<C_UInt32> &RLE_Lengths() { return RleChr.Lengths; }
 
 protected:
 	/// indexing chromosome
@@ -414,6 +417,8 @@ public:
 	void ResetChromosome();
 	/// return _Position which has been initialized
 	vector<C_Int32> &Position();
+	/// clear the buffer for variant positions
+	void ClearPosition();
 
 	/// return _GenoIndex which has been initialized
 	CGenoIndex &GenoIndex();
@@ -594,11 +599,11 @@ public:
 class COREARRAY_DLL_LOCAL CProgress
 {
 public:
-	CProgress(C_Int64 start, C_Int64 count, SEXP conn, bool newline);
-	virtual ~CProgress();
+	CProgress(C_Int64 count, SEXP Rconn, bool verbose);
+	~CProgress();
 
 	void Forward(C_Int64 Inc=1);
-	virtual void ShowProgress();
+	void ShowProgress();
 
 	inline C_Int64 Counter() const { return vCounter; }
 	inline C_Int64 TotalCount() const { return vTotalCount; }
@@ -606,27 +611,16 @@ public:
 protected:
 	C_Int64 vTotalCount;  ///< the total number
 	C_Int64 vCounter;     ///< the current counter
-	Rconnection File;     ///< R connection
+	Rconnection OutFile;  ///< R connection output
+	bool Verbose;         ///< whether display on stdout via Rprintf()
 	C_Int64 FwdCnt;       ///< the number of calling Forward()
 	time_t _start_time;   ///< the starting time
-	bool NewLine;
+	time_t _last_time;    ///< last saved time for calculating the interval
 	double _start, _step;
 	C_Int64 _hit;
 	vector< pair<double, time_t> > _timer;
 };
 
-class COREARRAY_DLL_LOCAL CProgressStdOut: public CProgress
-{
-public:
-	CProgressStdOut(C_Int64 count, int nproc, bool verbose);
-	virtual void ShowProgress();
-
-protected:
-	time_t _last_time;
-	int NProcess;
-	bool Verbose;
-};
-
 
 
 // ===========================================================
@@ -640,8 +634,16 @@ extern SEXP R_Data_Name;
 extern SEXP R_Data_Dim2_Name;
 extern SEXP R_Data_ListClass;
 
-extern int* R_Process_Count;
+// the index of child process
 extern int* R_Process_Index;
+// the number of child processes
+extern int* R_Process_Count;
+// list of file names for child processes
+extern vector<string> R_Process_StatusFName;
+// the index of block being used currently
+extern int R_Block_Index;
+// the number of blocks, 0 when it is not used
+extern int R_Block_Count;
 
 
 // ===========================================================
@@ -698,6 +700,9 @@ COREARRAY_DLL_LOCAL string GDS_PATH_PREFIX(const string &path, char prefix);
 /// output to a connection
 COREARRAY_DLL_LOCAL void ConnPutText(Rconnection file, const char *fmt, ...);
 
+/// time interval to string
+COREARRAY_DLL_LOCAL const char *time_str(double s);
+
 
 
 // ===========================================================
